/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 25, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.coredata.NSEntityMigrationPolicy;

import dstep.coredata.CoreDataDefines;
import dstep.coredata.NSEntityMapping;
import dstep.coredata.NSManagedObject;
import dstep.coredata.NSMigrationManager;
import dstep.foundation.NSArray;
import dstep.foundation.NSError;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

import bindings = dstep.coredata.NSEntityMigrationPolicy_bindings;

private
{
	NSString NSMigrationManagerKey_;
	NSString NSMigrationSourceObjectKey_;
	NSString NSMigrationDestinationObjectKey_;
	NSString NSMigrationEntityMappingKey_;
	NSString NSMigrationPropertyMappingKey_;
}

NSString NSMigrationManagerKey ()
{
	if (NSMigrationManagerKey_)
		return NSMigrationManagerKey_;

	return NSMigrationManagerKey_ = new NSString(bindings.NSMigrationManagerKey);
}

NSString NSMigrationSourceObjectKey ()
{
	if (NSMigrationSourceObjectKey_)
		return NSMigrationSourceObjectKey_;

	return NSMigrationSourceObjectKey_ = new NSString(bindings.NSMigrationSourceObjectKey);
}

NSString NSMigrationDestinationObjectKey ()
{
	if (NSMigrationDestinationObjectKey_)
		return NSMigrationDestinationObjectKey_;

	return NSMigrationDestinationObjectKey_ = new NSString(bindings.NSMigrationDestinationObjectKey);
}

NSString NSMigrationEntityMappingKey ()
{
	if (NSMigrationEntityMappingKey_)
		return NSMigrationEntityMappingKey_;

	return NSMigrationEntityMappingKey_ = new NSString(bindings.NSMigrationEntityMappingKey);
}

NSString NSMigrationPropertyMappingKey ()
{
	if (NSMigrationPropertyMappingKey_)
		return NSMigrationPropertyMappingKey_;

	return NSMigrationPropertyMappingKey_ = new NSString(bindings.NSMigrationPropertyMappingKey);
}

class NSEntityMigrationPolicy : NSObject
{
	mixin (ObjcWrap);

	bool beginEntityMapping (NSEntityMapping mapping, NSMigrationManager manager, ref NSError error)
	{
		id __arg2;

		if (error)
			__arg2 = error.objcObject;

		bool result = invokeObjcSelf!(bool, "beginEntityMapping:manager:error:", NSEntityMapping, NSMigrationManager, id*)(mapping, manager, &__arg2);

		if (__arg2)
			error = new NSError(__arg2);

		return result;
	}

	bool createDestinationInstancesForSourceInstance (NSManagedObject sInstance, NSEntityMapping mapping, NSMigrationManager manager, ref NSError error)
	{
		id __arg3;

		if (error)
			__arg3 = error.objcObject;

		bool result = invokeObjcSelf!(bool, "createDestinationInstancesForSourceInstance:entityMapping:manager:error:", NSManagedObject, NSEntityMapping, NSMigrationManager, id*)(sInstance, mapping, manager, &__arg3);

		if (__arg3)
			error = new NSError(__arg3);

		return result;
	}

	bool endInstanceCreationForEntityMapping (NSEntityMapping mapping, NSMigrationManager manager, ref NSError error)
	{
		id __arg2;

		if (error)
			__arg2 = error.objcObject;

		bool result = invokeObjcSelf!(bool, "endInstanceCreationForEntityMapping:manager:error:", NSEntityMapping, NSMigrationManager, id*)(mapping, manager, &__arg2);

		if (__arg2)
			error = new NSError(__arg2);

		return result;
	}

	bool createRelationshipsForDestinationInstance (NSManagedObject dInstance, NSEntityMapping mapping, NSMigrationManager manager, ref NSError error)
	{
		id __arg3;

		if (error)
			__arg3 = error.objcObject;

		bool result = invokeObjcSelf!(bool, "createRelationshipsForDestinationInstance:entityMapping:manager:error:", NSManagedObject, NSEntityMapping, NSMigrationManager, id*)(dInstance, mapping, manager, &__arg3);

		if (__arg3)
			error = new NSError(__arg3);

		return result;
	}

	bool endRelationshipCreationForEntityMapping (NSEntityMapping mapping, NSMigrationManager manager, ref NSError error)
	{
		id __arg2;

		if (error)
			__arg2 = error.objcObject;

		bool result = invokeObjcSelf!(bool, "endRelationshipCreationForEntityMapping:manager:error:", NSEntityMapping, NSMigrationManager, id*)(mapping, manager, &__arg2);

		if (__arg2)
			error = new NSError(__arg2);

		return result;
	}

	bool performCustomValidationForEntityMapping (NSEntityMapping mapping, NSMigrationManager manager, ref NSError error)
	{
		id __arg2;

		if (error)
			__arg2 = error.objcObject;

		bool result = invokeObjcSelf!(bool, "performCustomValidationForEntityMapping:manager:error:", NSEntityMapping, NSMigrationManager, id*)(mapping, manager, &__arg2);

		if (__arg2)
			error = new NSError(__arg2);

		return result;
	}

	bool endEntityMapping (NSEntityMapping mapping, NSMigrationManager manager, ref NSError error)
	{
		id __arg2;

		if (error)
			__arg2 = error.objcObject;

		bool result = invokeObjcSelf!(bool, "endEntityMapping:manager:error:", NSEntityMapping, NSMigrationManager, id*)(mapping, manager, &__arg2);

		if (__arg2)
			error = new NSError(__arg2);

		return result;
	}
}

