/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSArchiver;

import dstep.foundation.NSArray;
import dstep.foundation.NSCoder;
import dstep.foundation.NSData;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSString;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;



import bindings = dstep.foundation.NSArchiver_bindings;

private NSString NSInconsistentArchiveException_;

NSString NSInconsistentArchiveException ()
{
	if (NSInconsistentArchiveException_)
		return NSInconsistentArchiveException_;
	
	return NSInconsistentArchiveException_ = new NSString(bindings.NSInconsistentArchiveException);
}

const TNSArchiverCallback = `

	Class classForArchiver ()
	{
		return invokeObjcSelf!(Class, "classForArchiver");
	}
	
	Object replacementObjectForArchiver (NSArchiver archiver)
	{
		return invokeObjcSelf!(Object, "replacementObjectForArchiver:", NSArchiver)(archiver);
	}
	
	//mixin ObjcBindMethod!(classForArchiver, "classForArchiver");
	//mixin ObjcBindMethod!(replacementObjectForArchiver, "replacementObjectForArchiver:");
`;

class NSArchiver : NSCoder
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	Object initForWritingWithMutableData (NSMutableData mdata)
	{
		return invokeObjcSelf!(Object, "initForWritingWithMutableData:", NSMutableData)(mdata);
	}

	this (NSMutableData mdata)
	{
		typeof(this).alloc.initForWritingWithMutableData(mdata);
	}

	NSMutableData archiverData ()
	{
		return invokeObjcSelf!(NSMutableData, "archiverData");
	}

	void encodeRootObject (Object rootObject)
	{
		return invokeObjcSelf!(void, "encodeRootObject:", Object)(rootObject);
	}

	void encodeConditionalObject (Object object)
	{
		return invokeObjcSelf!(void, "encodeConditionalObject:", Object)(object);
	}

	static NSData archivedDataWithRootObject (Object rootObject)
	{
		return invokeObjcSuperClass!(NSData, "archivedDataWithRootObject:", Object)(rootObject);
	}

	static bool archiveRootObject (Object rootObject, NSString path)
	{
		return invokeObjcSuperClass!(bool, "archiveRootObject:toFile:", Object, NSString)(rootObject, path);
	}

	void encodeClassName (NSString trueName, NSString inArchiveName)
	{
		return invokeObjcSelf!(void, "encodeClassName:intoClassName:", NSString, NSString)(trueName, inArchiveName);
	}

	NSString classNameEncodedForTrueClassName (NSString trueName)
	{
		return invokeObjcSelf!(NSString, "classNameEncodedForTrueClassName:", NSString)(trueName);
	}

	void replaceObject (Object object, Object newObject)
	{
		return invokeObjcSelf!(void, "replaceObject:withObject:", Object, Object)(object, newObject);
	}
}

class NSUnarchiver : NSCoder
{
	mixin (ObjcWrap);	
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	Object initForReadingWithData (NSData data)
	{
		return invokeObjcSelf!(Object, "initForReadingWithData:", NSData)(data);
	}

	this (NSData data)
	{
		typeof(this).alloc.initForReadingWithData(data);
	}

	void setObjectZone (NSZone* zone)
	{
		return invokeObjcSelf!(void, "setObjectZone:", NSZone*)(zone);
	}

	NSZone* objectZone ()
	{
		return invokeObjcSelf!(NSZone*, "objectZone");
	}

	bool isAtEnd ()
	{
		return invokeObjcSelf!(bool, "isAtEnd");
	}

	uint systemVersion ()
	{
		return invokeObjcSelf!(uint, "systemVersion");
	}

	static Object unarchiveObjectWithData (NSData data)
	{
		return invokeObjcSuperClass!(Object, "unarchiveObjectWithData:", NSData)(data);
	}

	static Object unarchiveObjectWithFile (NSString path)
	{
		return invokeObjcSuperClass!(Object, "unarchiveObjectWithFile:", NSString)(path);
	}

	static void decodeClassName (NSString inArchiveName, NSString trueName)
	{
		return invokeObjcSuperClass!(void, "decodeClassName:asClassName:", NSString, NSString)(inArchiveName, trueName);
	}

	void decodeClassName (NSString inArchiveName, NSString trueName)
	{
		return invokeObjcSelf!(void, "decodeClassName:asClassName:", NSString, NSString)(inArchiveName, trueName);
	}

	static NSString classNameDecodedForArchiveClassName (NSString inArchiveName)
	{
		return invokeObjcSuperClass!(NSString, "classNameDecodedForArchiveClassName:", NSString)(inArchiveName);
	}

	NSString classNameDecodedForArchiveClassName (NSString inArchiveName)
	{
		return invokeObjcSelf!(NSString, "classNameDecodedForArchiveClassName:", NSString)(inArchiveName);
	}

	void replaceObject (Object object, Object newObject)
	{
		return invokeObjcSelf!(void, "replaceObject:withObject:", Object, Object)(object, newObject);
	}
}