/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSArray;

import dstep.foundation.NSCoder;
import dstep.foundation.NSData;
import dstep.foundation.NSEnumerator;
import dstep.foundation.NSIndexSet;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSPathUtilities;
import dstep.foundation.NSRange;
import dstep.foundation.NSSortDescriptor;
import dstep.foundation.NSString;
import dstep.foundation.NSURL;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

alias extern (C) NSInteger function (id, id, void*) CompareFunc;

const TNSArrayCreation = `

	static Object array ()
	{
		return invokeObjcSuperClass!(Object, "array");
	}

	static Object arrayWithObject (Object anObject)
	{
		return invokeObjcSuperClass!(Object, "arrayWithObject:", Object)(anObject);
	}

	static Object arrayWithObjects (id* objects, NSUInteger cnt)
	{
		return invokeObjcSuperClass!(Object, "arrayWithObjects:count:", id*, NSUInteger)(objects, cnt);
	}

	static Object arrayWithObjects (Object arrayWithObjects, ...)
	{
		return invokeObjcSuperClass!(Object, "arrayWithObjects:", Object)(arrayWithObjects);
	}

	static Object arrayWithArray (NSArray array)
	{
		return invokeObjcSuperClass!(Object, "arrayWithArray:", NSArray)(array);
	}

	Object initWithObjects (id* objects, NSUInteger cnt)
	{
		return invokeObjcSelf!(Object, "initWithObjects:count:", id*, NSUInteger)(objects, cnt);
	}

	this (id* objects, NSUInteger cnt)
	{
		typeof(this).alloc.initWithObjects(objects, cnt);
	}

	Object initWithObjects (Object initWithObjects, ...)
	{
		return invokeObjcSelf!(Object, "initWithObjects:", Object)(initWithObjects);
	}

	this (Object initWithObjects, ...)
	{
		typeof(this).alloc.initWithObjects(initWithObjects);
	}

	Object initWithArray (NSArray array)
	{
		return invokeObjcSelf!(Object, "initWithArray:", NSArray)(array);
	}

	this (NSArray array)
	{
		typeof(this).alloc.initWithArray(array);
	}

	Object initWithArray (NSArray array, bool flag)
	{
		return invokeObjcSelf!(Object, "initWithArray:copyItems:", NSArray, bool)(array, flag);
	}

	this (NSArray array, bool flag)
	{
		typeof(this).alloc.initWithArray(array, flag);
	}

	static Object arrayWithContentsOfFile (NSString path)
	{
		return invokeObjcSuperClass!(Object, "arrayWithContentsOfFile:", NSString)(path);
	}

	static Object arrayWithContentsOfURL (NSURL url)
	{
		return invokeObjcSuperClass!(Object, "arrayWithContentsOfURL:", NSURL)(url);
	}

	Object initWithContentsOfFile (NSString path)
	{
		return invokeObjcSelf!(Object, "initWithContentsOfFile:", NSString)(path);
	}

	this (NSString path)
	{
		typeof(this).alloc.initWithContentsOfFile(path);
	}

	Object initWithContentsOfURL (NSURL url)
	{
		return invokeObjcSelf!(Object, "initWithContentsOfURL:", NSURL)(url);
	}

	this (NSURL url)
	{
		typeof(this).alloc.initWithContentsOfURL(url);
	}
`;

const TNSExtendedArray = ` 

	NSArray arrayByAddingObject (Object anObject)
	{
		return invokeObjcSelf!(NSArray, "arrayByAddingObject:", Object)(anObject);
	}

	NSArray arrayByAddingObjectsFromArray (NSArray otherArray)
	{
		return invokeObjcSelf!(NSArray, "arrayByAddingObjectsFromArray:", NSArray)(otherArray);
	}

	NSString componentsJoinedByString (NSString separator)
	{
		return invokeObjcSelf!(NSString, "componentsJoinedByString:", NSString)(separator);
	}

	bool containsObject (Object anObject)
	{
		return invokeObjcSelf!(bool, "containsObject:", Object)(anObject);
	}

	NSString description ()
	{
		return invokeObjcSelf!(NSString, "description");
	}

	NSString descriptionWithLocale (Object locale)
	{
		return invokeObjcSelf!(NSString, "descriptionWithLocale:", Object)(locale);
	}

	NSString descriptionWithLocale (Object locale, NSUInteger level)
	{
		return invokeObjcSelf!(NSString, "descriptionWithLocale:indent:", Object, NSUInteger)(locale, level);
	}

	Object firstObjectCommonWithArray (NSArray otherArray)
	{
		return invokeObjcSelf!(Object, "firstObjectCommonWithArray:", NSArray)(otherArray);
	}

	void getObjects (id* objects)
	{
		return invokeObjcSelf!(void, "getObjects:", id*)(objects);
	}

	void getObjects (id* objects, NSRange range)
	{
		return invokeObjcSelf!(void, "getObjects:range:", id*, NSRange)(objects, range);
	}

	NSUInteger indexOfObject (Object anObject)
	{
		return invokeObjcSelf!(NSUInteger, "indexOfObject:", Object)(anObject);
	}

	NSUInteger indexOfObject (Object anObject, NSRange range)
	{
		return invokeObjcSelf!(NSUInteger, "indexOfObject:inRange:", Object, NSRange)(anObject, range);
	}

	NSUInteger indexOfObjectIdenticalTo (Object anObject)
	{
		return invokeObjcSelf!(NSUInteger, "indexOfObjectIdenticalTo:", Object)(anObject);
	}

	NSUInteger indexOfObjectIdenticalTo (Object anObject, NSRange range)
	{
		return invokeObjcSelf!(NSUInteger, "indexOfObjectIdenticalTo:inRange:", Object, NSRange)(anObject, range);
	}

	bool isEqualToArray (NSArray otherArray)
	{
		return invokeObjcSelf!(bool, "isEqualToArray:", NSArray)(otherArray);
	}

	Object lastObject ()
	{
		return invokeObjcSelf!(Object, "lastObject");
	}

	NSEnumerator objectEnumerator ()
	{
		return invokeObjcSelf!(NSEnumerator, "objectEnumerator");
	}

	NSEnumerator reverseObjectEnumerator ()
	{
		return invokeObjcSelf!(NSEnumerator, "reverseObjectEnumerator");
	}

	NSData sortedArrayHint ()
	{
		return invokeObjcSelf!(NSData, "sortedArrayHint");
	}

	NSArray sortedArrayUsingFunction (CompareFunc comparator, void* context)
	{
		return invokeObjcSelf!(NSArray, "sortedArrayUsingFunction:context:", CompareFunc, void*)(comparator, context);
	}

	NSArray sortedArrayUsingFunction (CompareFunc comparator, void* context, NSData hint)
	{
		return invokeObjcSelf!(NSArray, "sortedArrayUsingFunction:context:hint:", CompareFunc, void*, NSData)(comparator, context, hint);
	}

	NSArray sortedArrayUsingSelector (SEL comparator)
	{
		return invokeObjcSelf!(NSArray, "sortedArrayUsingSelector:", SEL)(comparator);
	}

	NSArray subarrayWithRange (NSRange range)
	{
		return invokeObjcSelf!(NSArray, "subarrayWithRange:", NSRange)(range);
	}

	bool writeToFile (NSString path, bool useAuxiliaryFile)
	{
		return invokeObjcSelf!(bool, "writeToFile:atomically:", NSString, bool)(path, useAuxiliaryFile);
	}

	bool writeToURL (NSURL url, bool atomically)
	{
		return invokeObjcSelf!(bool, "writeToURL:atomically:", NSURL, bool)(url, atomically);
	}

	void makeObjectsPerformSelector (SEL aSelector)
	{
		return invokeObjcSelf!(void, "makeObjectsPerformSelector:", SEL)(aSelector);
	}

	void makeObjectsPerformSelector (SEL aSelector, Object argument)
	{
		return invokeObjcSelf!(void, "makeObjectsPerformSelector:withObject:", SEL, Object)(aSelector, argument);
	}

	NSArray objectsAtIndexes (NSIndexSet indexes)
	{
		return invokeObjcSelf!(NSArray, "objectsAtIndexes:", NSIndexSet)(indexes);
	}
`;

const TNSExtendedMutableArray = `

	void addObjectsFromArray (NSArray otherArray)
	{
		return invokeObjcSelf!(void, "addObjectsFromArray:", NSArray)(otherArray);
	}

	void exchangeObjectAtIndex (NSUInteger idx1, NSUInteger idx2)
	{
		return invokeObjcSelf!(void, "exchangeObjectAtIndex:withObjectAtIndex:", NSUInteger, NSUInteger)(idx1, idx2);
	}

	void removeAllObjects ()
	{
		return invokeObjcSelf!(void, "removeAllObjects");
	}

	void removeObject (Object anObject, NSRange range)
	{
		return invokeObjcSelf!(void, "removeObject:inRange:", Object, NSRange)(anObject, range);
	}

	void removeObject (Object anObject)
	{
		return invokeObjcSelf!(void, "removeObject:", Object)(anObject);
	}

	void removeObjectIdenticalTo (Object anObject, NSRange range)
	{
		return invokeObjcSelf!(void, "removeObjectIdenticalTo:inRange:", Object, NSRange)(anObject, range);
	}

	void removeObjectIdenticalTo (Object anObject)
	{
		return invokeObjcSelf!(void, "removeObjectIdenticalTo:", Object)(anObject);
	}

	void removeObjectsFromIndices (NSUInteger* indices, NSUInteger cnt)
	{
		return invokeObjcSelf!(void, "removeObjectsFromIndices:numIndices:", NSUInteger*, NSUInteger)(indices, cnt);
	}

	void removeObjectsInArray (NSArray otherArray)
	{
		return invokeObjcSelf!(void, "removeObjectsInArray:", NSArray)(otherArray);
	}

	void removeObjectsInRange (NSRange range)
	{
		return invokeObjcSelf!(void, "removeObjectsInRange:", NSRange)(range);
	}

	void replaceObjectsInRange (NSRange range, NSArray otherArray, NSRange otherRange)
	{
		return invokeObjcSelf!(void, "replaceObjectsInRange:withObjectsFromArray:range:", NSRange, NSArray, NSRange)(range, otherArray, otherRange);
	}

	void replaceObjectsInRange (NSRange range, NSArray otherArray)
	{
		return invokeObjcSelf!(void, "replaceObjectsInRange:withObjectsFromArray:", NSRange, NSArray)(range, otherArray);
	}

	void setArray (NSArray otherArray)
	{
		return invokeObjcSelf!(void, "setArray:", NSArray)(otherArray);
	}

	void sortUsingFunction (CompareFunc compare, void* context)
	{
		return invokeObjcSelf!(void, "sortUsingFunction:context:", CompareFunc, void*)(compare, context);
	}

	void sortUsingSelector (SEL comparator)
	{
		return invokeObjcSelf!(void, "sortUsingSelector:", SEL)(comparator);
	}

	void insertObjects (NSArray objects, NSIndexSet indexes)
	{
		return invokeObjcSelf!(void, "insertObjects:atIndexes:", NSArray, NSIndexSet)(objects, indexes);
	}

	void removeObjectsAtIndexes (NSIndexSet indexes)
	{
		return invokeObjcSelf!(void, "removeObjectsAtIndexes:", NSIndexSet)(indexes);
	}

	void replaceObjectsAtIndexes (NSIndexSet indexes, NSArray objects)
	{
		return invokeObjcSelf!(void, "replaceObjectsAtIndexes:withObjects:", NSIndexSet, NSArray)(indexes, objects);
	}
`;

const TNSMutableArrayCreation = `

	static Object arrayWithCapacity (NSUInteger numItems)
	{
		return invokeObjcSuperClass!(Object, "arrayWithCapacity:", NSUInteger)(numItems);
	}

	Object initWithCapacity (NSUInteger numItems)
	{
		return invokeObjcSelf!(Object, "initWithCapacity:", NSUInteger)(numItems);
	}

	this (NSUInteger numItems)
	{
		typeof(this).alloc.initWithCapacity(numItems);
	}
`;

class NSMutableArray : NSArray
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	void addObject (Object anObject)
	{
		return invokeObjcSelf!(void, "addObject:", Object)(anObject);
	}

	void insertObject (Object anObject, NSUInteger index)
	{
		return invokeObjcSelf!(void, "insertObject:atIndex:", Object, NSUInteger)(anObject, index);
	}

	void removeLastObject ()
	{
		return invokeObjcSelf!(void, "removeLastObject");
	}

	void removeObjectAtIndex (NSUInteger index)
	{
		return invokeObjcSelf!(void, "removeObjectAtIndex:", NSUInteger)(index);
	}

	void replaceObjectAtIndex (NSUInteger index, Object anObject)
	{
		return invokeObjcSelf!(void, "replaceObjectAtIndex:withObject:", NSUInteger, Object)(index, anObject);
	}
	
	// NSExtendedMutableArray
	void addObjectsFromArray (NSArray otherArray)
	{
		return invokeObjcSelf!(void, "addObjectsFromArray:", NSArray)(otherArray);
	}
	
	void exchangeObjectAtIndex (NSUInteger idx1, NSUInteger idx2)
	{
		return invokeObjcSelf!(void, "exchangeObjectAtIndex:withObjectAtIndex:", NSUInteger, NSUInteger)(idx1, idx2);
	}
	
	void removeAllObjects ()
	{
		return invokeObjcSelf!(void, "removeAllObjects");
	}
	
	void removeObject (Object anObject, NSRange range)
	{
		return invokeObjcSelf!(void, "removeObject:inRange:", Object, NSRange)(anObject, range);
	}
	
	void removeObject (Object anObject)
	{
		return invokeObjcSelf!(void, "removeObject:", Object)(anObject);
	}
	
	void removeObjectIdenticalTo (Object anObject, NSRange range)
	{
		return invokeObjcSelf!(void, "removeObjectIdenticalTo:inRange:", Object, NSRange)(anObject, range);
	}
	
	void removeObjectIdenticalTo (Object anObject)
	{
		return invokeObjcSelf!(void, "removeObjectIdenticalTo:", Object)(anObject);
	}
	
	void removeObjectsFromIndices (NSUInteger* indices, NSUInteger cnt)
	{
		return invokeObjcSelf!(void, "removeObjectsFromIndices:numIndices:", NSUInteger*, NSUInteger)(indices, cnt);
	}
	
	void removeObjectsInArray (NSArray otherArray)
	{
		return invokeObjcSelf!(void, "removeObjectsInArray:", NSArray)(otherArray);
	}
	
	void removeObjectsInRange (NSRange range)
	{
		return invokeObjcSelf!(void, "removeObjectsInRange:", NSRange)(range);
	}
	
	void replaceObjectsInRange (NSRange range, NSArray otherArray, NSRange otherRange)
	{
		return invokeObjcSelf!(void, "replaceObjectsInRange:withObjectsFromArray:range:", NSRange, NSArray, NSRange)(range, otherArray, otherRange);
	}
	
	void replaceObjectsInRange (NSRange range, NSArray otherArray)
	{
		return invokeObjcSelf!(void, "replaceObjectsInRange:withObjectsFromArray:", NSRange, NSArray)(range, otherArray);
	}
	
	void setArray (NSArray otherArray)
	{
		return invokeObjcSelf!(void, "setArray:", NSArray)(otherArray);
	}
	
	void sortUsingFunction (CompareFunc compare, void* context)
	{
		return invokeObjcSelf!(void, "sortUsingFunction:context:", CompareFunc, void*)(compare, context);
	}
	
	void sortUsingSelector (SEL comparator)
	{
		return invokeObjcSelf!(void, "sortUsingSelector:", SEL)(comparator);
	}
	
	void insertObjects (NSArray objects, NSIndexSet indexes)
	{
		return invokeObjcSelf!(void, "insertObjects:atIndexes:", NSArray, NSIndexSet)(objects, indexes);
	}
	
	void removeObjectsAtIndexes (NSIndexSet indexes)
	{
		return invokeObjcSelf!(void, "removeObjectsAtIndexes:", NSIndexSet)(indexes);
	}
	
	void replaceObjectsAtIndexes (NSIndexSet indexes, NSArray objects)
	{
		return invokeObjcSelf!(void, "replaceObjectsAtIndexes:withObjects:", NSIndexSet, NSArray)(indexes, objects);
	}
	
	
	// NSMutableArrayCreation
	static Object arrayWithCapacity (NSUInteger numItems)
	{
		return invokeObjcSuperClass!(Object, "arrayWithCapacity:", NSUInteger)(numItems);
	}
	
	Object initWithCapacity (NSUInteger numItems)
	{
		return invokeObjcSelf!(Object, "initWithCapacity:", NSUInteger)(numItems);
	}
	
	this (NSUInteger numItems)
	{
		typeof(this).alloc.initWithCapacity(numItems);
	}
}

class NSArray : NSObject, INSCopying, INSMutableCopying, INSCoding, INSFastEnumeration
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	NSUInteger count ()
	{
		return invokeObjcSelf!(NSUInteger, "count");
	}

	Object objectAtIndex (NSUInteger index)
	{
		return invokeObjcSelf!(Object, "objectAtIndex:", NSUInteger)(index);
	}

	Object copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "copyWithZone:", NSZone*)(zone);
	}

	Object mutableCopyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "mutableCopyWithZone:", NSZone*)(zone);
	}

	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}

	Object initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(Object, "initWithCoder:", NSCoder)(aDecoder);
	}

	this (NSCoder aDecoder)
	{
		typeof(this).alloc.initWithCoder(aDecoder);
	}

	NSUInteger countByEnumeratingWithState (NSFastEnumerationState* state, id* stackbuf, NSUInteger len)
	{
		return invokeObjcSelf!(NSUInteger, "countByEnumeratingWithState:objects:count:", NSFastEnumerationState*, id*, NSUInteger)(state, stackbuf, len);
	}
	
	// NSArrayCreation
	static Object array ()
	{
		return invokeObjcSuperClass!(Object, "array");
	}
	
	static Object arrayWithObject (Object anObject)
	{
		return invokeObjcSuperClass!(Object, "arrayWithObject:", Object)(anObject);
	}
	
	static Object arrayWithObjects (id* objects, NSUInteger cnt)
	{
		return invokeObjcSuperClass!(Object, "arrayWithObjects:count:", id*, NSUInteger)(objects, cnt);
	}
	
	static Object arrayWithObjects (Object arrayWithObjects, ...)
	{
		return invokeObjcSuperClass!(Object, "arrayWithObjects:", Object)(arrayWithObjects);
	}
	
	static Object arrayWithArray (NSArray array)
	{
		return invokeObjcSuperClass!(Object, "arrayWithArray:", NSArray)(array);
	}
	
	Object initWithObjects (id* objects, NSUInteger cnt)
	{
		return invokeObjcSelf!(Object, "initWithObjects:count:", id*, NSUInteger)(objects, cnt);
	}
	
	this (id* objects, NSUInteger cnt)
	{
		typeof(this).alloc.initWithObjects(objects, cnt);
	}
	
	Object initWithObjects (Object initWithObjects, ...)
	{
		return invokeObjcSelf!(Object, "initWithObjects:", Object)(initWithObjects);
	}
	
	this (Object initWithObjects, ...)
	{
		typeof(this).alloc.initWithObjects(initWithObjects);
	}
	
	Object initWithArray (NSArray array)
	{
		return invokeObjcSelf!(Object, "initWithArray:", NSArray)(array);
	}
	
	this (NSArray array)
	{
		typeof(this).alloc.initWithArray(array);
	}
	
	Object initWithArray (NSArray array, bool flag)
	{
		return invokeObjcSelf!(Object, "initWithArray:copyItems:", NSArray, bool)(array, flag);
	}
	
	this (NSArray array, bool flag)
	{
		typeof(this).alloc.initWithArray(array, flag);
	}
	
	static Object arrayWithContentsOfFile (NSString path)
	{
		return invokeObjcSuperClass!(Object, "arrayWithContentsOfFile:", NSString)(path);
	}
	
	static Object arrayWithContentsOfURL (NSURL url)
	{
		return invokeObjcSuperClass!(Object, "arrayWithContentsOfURL:", NSURL)(url);
	}
	
	Object initWithContentsOfFile (NSString path)
	{
		return invokeObjcSelf!(Object, "initWithContentsOfFile:", NSString)(path);
	}
	
	this (NSString path)
	{
		typeof(this).alloc.initWithContentsOfFile(path);
	}
	
	Object initWithContentsOfURL (NSURL url)
	{
		return invokeObjcSelf!(Object, "initWithContentsOfURL:", NSURL)(url);
	}
	
	this (NSURL url)
	{
		typeof(this).alloc.initWithContentsOfURL(url);
	}
	
	// NSExtendedArray
	NSArray arrayByAddingObject (Object anObject)
	{
		return invokeObjcSelf!(NSArray, "arrayByAddingObject:", Object)(anObject);
	}
	
	NSArray arrayByAddingObjectsFromArray (NSArray otherArray)
	{
		return invokeObjcSelf!(NSArray, "arrayByAddingObjectsFromArray:", NSArray)(otherArray);
	}
	
	NSString componentsJoinedByString (NSString separator)
	{
		return invokeObjcSelf!(NSString, "componentsJoinedByString:", NSString)(separator);
	}
	
	bool containsObject (Object anObject)
	{
		return invokeObjcSelf!(bool, "containsObject:", Object)(anObject);
	}
	
	NSString description ()
	{
		return invokeObjcSelf!(NSString, "description");
	}
	
	NSString descriptionWithLocale (Object locale)
	{
		return invokeObjcSelf!(NSString, "descriptionWithLocale:", Object)(locale);
	}
	
	NSString descriptionWithLocale (Object locale, NSUInteger level)
	{
		return invokeObjcSelf!(NSString, "descriptionWithLocale:indent:", Object, NSUInteger)(locale, level);
	}
	
	Object firstObjectCommonWithArray (NSArray otherArray)
	{
		return invokeObjcSelf!(Object, "firstObjectCommonWithArray:", NSArray)(otherArray);
	}
	
	void getObjects (id* objects)
	{
		return invokeObjcSelf!(void, "getObjects:", id*)(objects);
	}
	
	void getObjects (id* objects, NSRange range)
	{
		return invokeObjcSelf!(void, "getObjects:range:", id*, NSRange)(objects, range);
	}
	
	NSUInteger indexOfObject (Object anObject)
	{
		return invokeObjcSelf!(NSUInteger, "indexOfObject:", Object)(anObject);
	}
	
	NSUInteger indexOfObject (Object anObject, NSRange range)
	{
		return invokeObjcSelf!(NSUInteger, "indexOfObject:inRange:", Object, NSRange)(anObject, range);
	}
	
	NSUInteger indexOfObjectIdenticalTo (Object anObject)
	{
		return invokeObjcSelf!(NSUInteger, "indexOfObjectIdenticalTo:", Object)(anObject);
	}
	
	NSUInteger indexOfObjectIdenticalTo (Object anObject, NSRange range)
	{
		return invokeObjcSelf!(NSUInteger, "indexOfObjectIdenticalTo:inRange:", Object, NSRange)(anObject, range);
	}
	
	bool isEqualToArray (NSArray otherArray)
	{
		return invokeObjcSelf!(bool, "isEqualToArray:", NSArray)(otherArray);
	}
	
	Object lastObject ()
	{
		return invokeObjcSelf!(Object, "lastObject");
	}
	
	NSEnumerator objectEnumerator ()
	{
		return invokeObjcSelf!(NSEnumerator, "objectEnumerator");
	}
	
	NSEnumerator reverseObjectEnumerator ()
	{
		return invokeObjcSelf!(NSEnumerator, "reverseObjectEnumerator");
	}
	
	NSData sortedArrayHint ()
	{
		return invokeObjcSelf!(NSData, "sortedArrayHint");
	}
	
	NSArray sortedArrayUsingFunction (CompareFunc comparator, void* context)
	{
		return invokeObjcSelf!(NSArray, "sortedArrayUsingFunction:context:", CompareFunc, void*)(comparator, context);
	}
	
	NSArray sortedArrayUsingFunction (CompareFunc comparator, void* context, NSData hint)
	{
		return invokeObjcSelf!(NSArray, "sortedArrayUsingFunction:context:hint:", CompareFunc, void*, NSData)(comparator, context, hint);
	}
	
	NSArray sortedArrayUsingSelector (SEL comparator)
	{
		return invokeObjcSelf!(NSArray, "sortedArrayUsingSelector:", SEL)(comparator);
	}
	
	NSArray subarrayWithRange (NSRange range)
	{
		return invokeObjcSelf!(NSArray, "subarrayWithRange:", NSRange)(range);
	}
	
	bool writeToFile (NSString path, bool useAuxiliaryFile)
	{
		return invokeObjcSelf!(bool, "writeToFile:atomically:", NSString, bool)(path, useAuxiliaryFile);
	}
	
	bool writeToURL (NSURL url, bool atomically)
	{
		return invokeObjcSelf!(bool, "writeToURL:atomically:", NSURL, bool)(url, atomically);
	}
	
	void makeObjectsPerformSelector (SEL aSelector)
	{
		return invokeObjcSelf!(void, "makeObjectsPerformSelector:", SEL)(aSelector);
	}
	
	void makeObjectsPerformSelector (SEL aSelector, Object argument)
	{
		return invokeObjcSelf!(void, "makeObjectsPerformSelector:withObject:", SEL, Object)(aSelector, argument);
	}
	
	NSArray objectsAtIndexes (NSIndexSet indexes)
	{
		return invokeObjcSelf!(NSArray, "objectsAtIndexes:", NSIndexSet)(indexes);
	}
	
	// NSSortDescriptorSorting
	NSArray sortedArrayUsingDescriptors (NSArray sortDescriptors)
	{
		return invokeObjcSelf!(NSArray, "sortedArrayUsingDescriptors:", NSArray)(sortDescriptors);
	}
	
	// NSArrayPathExtensions
	NSArray pathsMatchingExtensions (NSArray filterTypes)
	{
		return invokeObjcSelf!(NSArray, "pathsMatchingExtensions:", NSArray)(filterTypes);
	}
}