/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSScriptStandardSuiteCommands;

import dstep.foundation.NSDictionary;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSScriptClassDescription;
import dstep.foundation.NSScriptCommand;
import dstep.foundation.NSScriptObjectSpecifiers;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

alias NSUInteger NSSaveOptions;

enum
{
	NSSaveOptionsYes = 0,
	NSSaveOptionsNo,
	NSSaveOptionsAsk
}

class NSCloneCommand : NSScriptCommand
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	void setReceiversSpecifier (NSScriptObjectSpecifier receiversRef)
	{
		return invokeObjcSelf!(void, "setReceiversSpecifier:", NSScriptObjectSpecifier)(receiversRef);
	}

	NSScriptObjectSpecifier keySpecifier ()
	{
		return invokeObjcSelf!(NSScriptObjectSpecifier, "keySpecifier");
	}
}

class NSSetCommand : NSScriptCommand
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}
	
	void setReceiversSpecifier (NSScriptObjectSpecifier receiversRef)
	{
		return invokeObjcSelf!(void, "setReceiversSpecifier:", NSScriptObjectSpecifier)(receiversRef);
	}

	NSScriptObjectSpecifier keySpecifier ()
	{
		return invokeObjcSelf!(NSScriptObjectSpecifier, "keySpecifier");
	}
}

class NSCreateCommand : NSScriptCommand
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	NSScriptClassDescription createClassDescription ()
	{
		return invokeObjcSelf!(NSScriptClassDescription, "createClassDescription");
	}

	NSDictionary resolvedKeyDictionary ()
	{
		return invokeObjcSelf!(NSDictionary, "resolvedKeyDictionary");
	}
}

class NSMoveCommand : NSScriptCommand
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	void setReceiversSpecifier (NSScriptObjectSpecifier receiversRef)
	{
		return invokeObjcSelf!(void, "setReceiversSpecifier:", NSScriptObjectSpecifier)(receiversRef);
	}

	NSScriptObjectSpecifier keySpecifier ()
	{
		return invokeObjcSelf!(NSScriptObjectSpecifier, "keySpecifier");
	}
}

class NSQuitCommand : NSScriptCommand
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	uint saveOptions ()
	{
		return invokeObjcSelf!(uint, "saveOptions");
	}
}

class NSCloseCommand : NSScriptCommand
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	uint saveOptions ()
	{
		return invokeObjcSelf!(uint, "saveOptions");
	}
}

class NSDeleteCommand : NSScriptCommand
{
	mixin (ObjcWrap);
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	void setReceiversSpecifier (NSScriptObjectSpecifier receiversRef)
	{
		return invokeObjcSelf!(void, "setReceiversSpecifier:", NSScriptObjectSpecifier)(receiversRef);
	}

	NSScriptObjectSpecifier keySpecifier ()
	{
		return invokeObjcSelf!(NSScriptObjectSpecifier, "keySpecifier");
	}
}

